/*
* CapMeterSimple.cpp
*
* Created: 11.06.2016 09:56:48
* Author : kner
* Button starts measurement: discharge the C, then release ist to be charged by a known Pullup; after a fixed time of 10ms
* get the value of the voltage by the built-in ADC; display this value
* display: 3 digit 7seg multiplexed

Original Aufgabenstellung:
AVR Projekt inkl. Simulation und Kommentaren; die exakten Zeiten sind anzugeben.

Gemultiplexte Siebensegmentanzeige: der 7Segmentcode der Elemente wird in den Variablen D1,D2,D3 abgespeichert.
An PortC werden die Daten ausgegeben,
am PortB werden nacheinander die Siebensegmente kurz eingeschaltet, sodass ein stehendes Bild auf der Anzeige erscheint.
Die Multiplexsignale Arduino2 bis 4 werden ber den Timer 1 Interrupt erzeugt (Takt ca. 1ms) .
Am Port B wird also nacheinander 001 010 100 001 010 100 001.  usw ausgegeben.
Wenn am Arduino2 auf PortB eine 1 liegt muss also am PortC das Datum D3 anliegen.

Das Unterprogramm Bin27seg wandelt ein Nibble in einen 7Segmentwert von '0' bis '9'

Der Kondensator wird ber  den Widerstand geladen und liegt am ADU.

Die Taste T startet eine Messung:
1. der Kondensator wird entladen, ein Timer auf Null gestellt
2. der Kondensator darf sich aufladen, die Zeitmessung wird gestartet
3. der Kondensator ldt fr 10ms, dann wird der analoge Spannungswert am C eingelesen (8 Bit)
4. dieser Zahlenwert wird in seine 3 Stellen zerlegt, in 7Seg Code umgewandelt und am Display ausgegeben



*/

#define F_CPU 16000000U
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define BUTTON 0
#define DISCHARGEPIN 1

uint8_t D3;  // digit 3 on Display 3 shows 7seg
uint8_t D2;
uint8_t D1;


void setup();
uint8_t bin27seg(uint8_t x); //binary code to 7seg code

int main(void)
{
	setup();
	while (1)
	{
		// wait for button
		loop_until_bit_is_clear(PIND,BUTTON); 
		// start measurement		
		// discharge
		PORTD &=~(1<<DISCHARGEPIN); // set the pin to 0
		DDRD  |= (1<<DISCHARGEPIN); // set the pin to Ouput - now the pin drives 0V
		
		// wait 100ms to discharge
		_delay_ms(100);
		// start loading		
		DDRD  &= ~(1<<DISCHARGEPIN); // set the pin to Input - now the pin is high impedance
		// wait 10ms
		_delay_ms(10);

		// get ADC Value
		ADCSRA |= (1<<ADSC); // start single conversion
		loop_until_bit_is_clear(ADCSRA,ADSC); // wait for conversion to complete
		uint8_t data = ADCH;
		// test it
		data = 123;
		// get HZE (hunderter zehner einer) and display
		uint8_t h = data / 100;
		uint8_t rest = data - (data / 100)*100;
		uint8_t t = rest / 10;
		uint8_t o = rest % 10;
		//Convert to 7seg
		D3 = bin27seg(h);
		D2 = bin27seg(t);
		D1 = bin27seg(o);		
	}
}

void setup(){
	//DISPLAY
	DDRC = 0xff; //data port
	DDRB = 0x07; //lower 3 lines for mux signals
	
	// external button with pullup
	PORTD |= (1<<BUTTON);
	// discharge pin
	DDRD |= (1<<DISCHARGEPIN);
	// ADC input ADC7
	ADMUX = 7;
	// ADC single conversion
	// enable adc, set referene to AVCC
	ADMUX |= (1<<REFS0); //AVCC
	ADCSRA = (1<<ADEN);
	ADMUX |= (1<<ADLAR); //left adjust - we only need 8 bit result
	
	ADCSRA &= ~(1<<ADATE); // single conversion!
	ADCSRA = (1<<ADSC); //Start one conversion for init
	
	//periodic timer1 1ms for display
	//Autoreload Mode  16MHz => 62.5ns => 1ms = 16000  => Autoreload value = 65536-16000=49536
	TCNT1 = 49536;
	// Start timer1
	TCCR1B = 1;
	// Enable Timer1 Overflow Interrupt
	TIMSK1 |= (1<< TOIE1);
	
	sei();
}
ISR (TIMER1_OVF_vect){  //periodic 1ms for display
	TCNT1 = 49536; // not necessary to stop the timer for reload
	volatile static uint8_t display_number;
	display_number++;
	display_number = display_number % 3;
	switch (display_number){
		case 0:
		PORTC = D3;
		PORTB = 0b00000100;
		break;
		case 1:
		PORTC = D2;
		PORTB = 0b00000010;
		break;
		case 2:
		PORTC = D1;
		PORTB = 0b00000001;
		break;
	}
}


uint8_t bin27seg(uint8_t x){
	uint8_t code;
	switch (x) {
		case 0:
		code=0b00000001;
		break;
		case 1:
		code=0b00000010;
		break;
		case 2:
		code=0b00000011;
		break;
		default:
		code=0b11111111;		
	}
	return code;
}
